import { XrplNetwork, Wallet } from 'xpring-common-js';
import { GrpcNetworkClientInterface } from './network-clients/grpc-network-client-interface';
import TrustLine from './shared/trustline';
import { JsonNetworkClientInterface } from './network-clients/json-network-client-interface';
import TransactionResult from './shared/transaction-result';
/**
 * IssuedCurrencyClient is a client for working with Issued Currencies on the XRPL.
 * @see https://xrpl.org/issued-currencies-overview.html
 */
export default class IssuedCurrencyClient {
    private readonly jsonNetworkClient;
    readonly network: XrplNetwork;
    private coreXrplClient;
    /**
     * Create a new IssuedCurrencyClient.
     *
     * The IssuedCurrencyClient will use gRPC to communicate with the given endpoint.
     *
     * @param grpcUrl The URL of the gRPC instance to connect to.
     * @param network The network this IssuedCurrencyClient is connecting to.
     * @param forceWeb If `true`, then we will use the gRPC-Web client even when on Node. Defaults to false. This is mainly for testing and in the future will be removed when we have browser testing.
     */
    static issuedCurrencyClientWithEndpoint(grpcUrl: string, jsonUrl: string, network: XrplNetwork, forceWeb?: boolean): IssuedCurrencyClient;
    /**
     * Create a new IssuedCurrencyClient with a custom network client implementation.
     *
     * In general, clients should prefer to call `issuedCurrencyClientWithEndpoint`. This constructor is provided to improve testability of this class.
     *
     * @param networkClient A network client which will manage remote RPCs to Rippled.
     * @param network The network this IssuedCurrencyClient is connecting to.
     */
    constructor(grpcNetworkClient: GrpcNetworkClientInterface, jsonNetworkClient: JsonNetworkClientInterface, network: XrplNetwork);
    /**
     * Retrieves information about an account's trust lines, which maintain balances of all non-XRP currencies and assets.
     * @see https://xrpl.org/trust-lines-and-issuing.html
     *
     * @param account The account for which to retrieve associated trust lines, encoded as an X-Address.
     * @see https://xrpaddress.info/
     * @returns An array of TrustLine objects, representing all trust lines associated with this account.
     */
    getTrustLines(account: string): Promise<Array<TrustLine>>;
    /**
     * Enable Require Authorization for this XRPL account.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#require-auth
     *
     * @param wallet The wallet associated with the XRPL account enabling Require Authorization and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    requireAuthorizedTrustlines(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Disable Require Authorization for this XRPL account.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#require-auth
     *
     * @param wallet The wallet associated with the XRPL account disabling Require Authorization and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    allowUnauthorizedTrustlines(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Enable Default Ripple for this XRPL account.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#default-ripple
     *
     * @param wallet The wallet associated with the XRPL account enabling Default Ripple and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    enableRippling(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Enable Disallow XRP for this XRPL account.
     * Note that the meaning of this flag is not enforced by rippled, and is only intended for use by client applications.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#disallow-xrp
     *
     * @param wallet The wallet associated with the XRPL account enabling Disallow XRP and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    disallowIncomingXrp(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Disable Disallow XRP for this XRPL account.
     * Note that the meaning of this flag is not enforced by rippled, and is only intended for use by client applications.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#disallow-xrp
     *
     * @param wallet The wallet associated with the XRPL account disabling Disallow XRP and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    allowIncomingXrp(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Enable Require Destination Tags for this XRPL account.
     *
     * @see https://xrpl.org/require-destination-tags.html
     *
     * @param wallet The wallet associated with the XRPL account enabling Require Destination Tags and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    requireDestinationTags(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Disable Require Destination for this XRPL account.
     *
     * @see https://xrpl.org/require-destination-tags.html
     *
     * @param wallet The wallet associated with the XRPL account disabling Require Destination and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    allowNoDestinationTag(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Set the Transfer Fees for an issuing account.
     * The Transfer Fee is a percentage to charge when two users transfer an issuer's IOUs on the XRPL.
     *
     * @see https://xrpl.org/transfer-fees.html
     *
     * @param transferFee The amount you must send for the recipient to get 1 billion units of the same currency.
     *                    It cannot be set to less than 1000000000 or more than 2000000000.
     * @param wallet The wallet associated with the issuing account, and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    setTransferFee(transferFee: number, wallet: Wallet): Promise<TransactionResult>;
    /**
     * Enable Global Freeze for this XRPL account.
     *
     * @see https://xrpl.org/freezes.html#global-freeze
     *
     * @param wallet The wallet associated with the XRPL account enabling Global Freeze and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    enableGlobalFreeze(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Disable Global Freeze for this XRPL account.
     *
     * @see https://xrpl.org/freezes.html#global-freeze
     *
     * @param wallet The wallet associated with the XRPL account disabling Global Freeze and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    disableGlobalFreeze(wallet: Wallet): Promise<TransactionResult>;
    /**
     * Permanently enable No Freeze for this XRPL account.
     *
     * @see https://xrpl.org/freezes.html#no-freeze
     *
     * @param wallet The wallet associated with the XRPL account enabling No Freeze and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    enableNoFreeze(wallet: Wallet): Promise<TransactionResult>;
}
