"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const xpring_common_js_1 = require("xpring-common-js");
const grpc_xrp_network_client_1 = __importDefault(require("./network-clients/grpc-xrp-network-client"));
const grpc_xrp_network_client_web_1 = __importDefault(require("./network-clients/grpc-xrp-network-client.web"));
const utils_1 = __importDefault(require("../Common/utils"));
const core_xrpl_client_1 = __importDefault(require("./core-xrpl-client"));
const trustline_1 = __importDefault(require("./shared/trustline"));
const json_rpc_network_client_1 = __importDefault(require("./network-clients/json-rpc-network-client"));
const shared_1 = require("./shared");
const account_set_flag_1 = require("./shared/account-set-flag");
const common_pb_1 = require("./Generated/node/org/xrpl/rpc/v1/common_pb");
const transaction_pb_1 = require("./Generated/node/org/xrpl/rpc/v1/transaction_pb");
/**
 * IssuedCurrencyClient is a client for working with Issued Currencies on the XRPL.
 * @see https://xrpl.org/issued-currencies-overview.html
 */
class IssuedCurrencyClient {
    /**
     * Create a new IssuedCurrencyClient with a custom network client implementation.
     *
     * In general, clients should prefer to call `issuedCurrencyClientWithEndpoint`. This constructor is provided to improve testability of this class.
     *
     * @param networkClient A network client which will manage remote RPCs to Rippled.
     * @param network The network this IssuedCurrencyClient is connecting to.
     */
    constructor(grpcNetworkClient, jsonNetworkClient, network) {
        this.jsonNetworkClient = jsonNetworkClient;
        this.network = network;
        this.coreXrplClient = new core_xrpl_client_1.default(grpcNetworkClient, network);
    }
    /**
     * Create a new IssuedCurrencyClient.
     *
     * The IssuedCurrencyClient will use gRPC to communicate with the given endpoint.
     *
     * @param grpcUrl The URL of the gRPC instance to connect to.
     * @param network The network this IssuedCurrencyClient is connecting to.
     * @param forceWeb If `true`, then we will use the gRPC-Web client even when on Node. Defaults to false. This is mainly for testing and in the future will be removed when we have browser testing.
     */
    static issuedCurrencyClientWithEndpoint(grpcUrl, jsonUrl, network, forceWeb = false) {
        const grpcNetworkClient = utils_1.default() && !forceWeb
            ? new grpc_xrp_network_client_1.default(grpcUrl)
            : new grpc_xrp_network_client_web_1.default(grpcUrl);
        return new IssuedCurrencyClient(grpcNetworkClient, new json_rpc_network_client_1.default(jsonUrl), network);
    }
    /**
     * Retrieves information about an account's trust lines, which maintain balances of all non-XRP currencies and assets.
     * @see https://xrpl.org/trust-lines-and-issuing.html
     *
     * @param account The account for which to retrieve associated trust lines, encoded as an X-Address.
     * @see https://xrpaddress.info/
     * @returns An array of TrustLine objects, representing all trust lines associated with this account.
     */
    async getTrustLines(account) {
        const classicAddress = xpring_common_js_1.XrpUtils.decodeXAddress(account);
        if (!classicAddress) {
            throw shared_1.XrpError.xAddressRequired;
        }
        const accountLinesResponse = await this.jsonNetworkClient.getAccountLines(classicAddress.address);
        if (accountLinesResponse.result.error) {
            throw shared_1.XrpError.accountNotFound;
        }
        const rawTrustLines = accountLinesResponse.result.lines;
        if (rawTrustLines === undefined) {
            throw shared_1.XrpError.malformedResponse;
        }
        const trustLines = [];
        rawTrustLines.map((trustLineJson) => {
            trustLines.push(new trustline_1.default(trustLineJson));
        });
        return trustLines;
    }
    /**
     * Enable Require Authorization for this XRPL account.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#require-auth
     *
     * @param wallet The wallet associated with the XRPL account enabling Require Authorization and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async requireAuthorizedTrustlines(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfRequireAuth, true, wallet);
    }
    /**
     * Disable Require Authorization for this XRPL account.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#require-auth
     *
     * @param wallet The wallet associated with the XRPL account disabling Require Authorization and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async allowUnauthorizedTrustlines(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfRequireAuth, false, wallet);
    }
    /**
     * Enable Default Ripple for this XRPL account.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#default-ripple
     *
     * @param wallet The wallet associated with the XRPL account enabling Default Ripple and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async enableRippling(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfDefaultRipple, true, wallet);
    }
    /**
     * Enable Disallow XRP for this XRPL account.
     * Note that the meaning of this flag is not enforced by rippled, and is only intended for use by client applications.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#disallow-xrp
     *
     * @param wallet The wallet associated with the XRPL account enabling Disallow XRP and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async disallowIncomingXrp(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfDisallowXRP, true, wallet);
    }
    /**
     * Disable Disallow XRP for this XRPL account.
     * Note that the meaning of this flag is not enforced by rippled, and is only intended for use by client applications.
     *
     * @see https://xrpl.org/become-an-xrp-ledger-gateway.html#disallow-xrp
     *
     * @param wallet The wallet associated with the XRPL account disabling Disallow XRP and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async allowIncomingXrp(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfDisallowXRP, false, wallet);
    }
    /**
     * Enable Require Destination Tags for this XRPL account.
     *
     * @see https://xrpl.org/require-destination-tags.html
     *
     * @param wallet The wallet associated with the XRPL account enabling Require Destination Tags and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async requireDestinationTags(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfRequireDest, true, wallet);
    }
    /**
     * Disable Require Destination for this XRPL account.
     *
     * @see https://xrpl.org/require-destination-tags.html
     *
     * @param wallet The wallet associated with the XRPL account disabling Require Destination and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async allowNoDestinationTag(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfRequireDest, false, wallet);
    }
    /**
     * Set the Transfer Fees for an issuing account.
     * The Transfer Fee is a percentage to charge when two users transfer an issuer's IOUs on the XRPL.
     *
     * @see https://xrpl.org/transfer-fees.html
     *
     * @param transferFee The amount you must send for the recipient to get 1 billion units of the same currency.
     *                    It cannot be set to less than 1000000000 or more than 2000000000.
     * @param wallet The wallet associated with the issuing account, and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async setTransferFee(transferFee, wallet) {
        const transferRate = new common_pb_1.TransferRate();
        transferRate.setValue(transferFee);
        const accountSet = new transaction_pb_1.AccountSet();
        accountSet.setTransferRate(transferRate);
        const transaction = await this.coreXrplClient.prepareBaseTransaction(wallet);
        transaction.setAccountSet(accountSet);
        const transactionHash = await this.coreXrplClient.signAndSubmitTransaction(transaction, wallet);
        return await this.coreXrplClient.getFinalTransactionResultAsync(transactionHash, wallet);
    }
    /**
     * Enable Global Freeze for this XRPL account.
     *
     * @see https://xrpl.org/freezes.html#global-freeze
     *
     * @param wallet The wallet associated with the XRPL account enabling Global Freeze and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async enableGlobalFreeze(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfGlobalFreeze, true, wallet);
    }
    /**
     * Disable Global Freeze for this XRPL account.
     *
     * @see https://xrpl.org/freezes.html#global-freeze
     *
     * @param wallet The wallet associated with the XRPL account disabling Global Freeze and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async disableGlobalFreeze(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfGlobalFreeze, false, wallet);
    }
    /**
     * Permanently enable No Freeze for this XRPL account.
     *
     * @see https://xrpl.org/freezes.html#no-freeze
     *
     * @param wallet The wallet associated with the XRPL account enabling No Freeze and that will sign the request.
     * @returns A promise which resolves to a TransactionResult object that represents the result of this transaction.
     */
    async enableNoFreeze(wallet) {
        return this.coreXrplClient.changeFlag(account_set_flag_1.AccountSetFlag.asfNoFreeze, true, wallet);
    }
}
exports.default = IssuedCurrencyClient;
//# sourceMappingURL=issued-currency-client.js.map