"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
/**
 * A network client for interacting with the rippled JSON RPC.
 * @see https://xrpl.org/rippled-api.html
 */
class JsonRpcNetworkClient {
    /**
     * Create a new JsonRpcNetworkClient.
     *
     * @param jsonUrl The URL of the rippled node to query.
     * @see https://xrpl.org/get-started-with-the-rippled-api.html
     */
    constructor(jsonUrl) {
        this.axiosInstance = axios_1.default.create({
            baseURL: jsonUrl,
        });
    }
    /**
     * Submit a JSON request to the rippled server.
     *
     * @param jsonRequest A JSON object specifying the method and params of the request.
     * @see https://xrpl.org/request-formatting.html#json-rpc-format
     * @returns The response from the rippled server.
     */
    async submitRequest(jsonRequest) {
        const axiosResponse = await this.axiosInstance
            .request({
            url: '/',
            method: 'post',
            data: jsonRequest,
            headers: { 'Content-Type': 'application/json' },
        })
            .catch(function (error) {
            if (error.response) {
                // The request was made and the server responded with a status code
                // that falls out of the range of 2xx
                throw new Error(error.toJSON());
            }
            else if (error.request) {
                // The request was made but no response was received
                // `error.request` is an instance of XMLHttpRequest in the browser and an instance of
                // http.ClientRequest in node.js
                throw new Error('The request was made but no response was received.');
            }
            else {
                // Something happened in setting up the request that triggered an Error
                throw new Error(error.message);
            }
        });
        return axiosResponse;
    }
    /**
     * Submits an account_lines request to the rippled JSON RPC.
     *
     * @param account The XRPL account to query for trust lines.
     */
    async getAccountLines(account) {
        // TODO: consider an option for including the 'peer' param in the request, which limits the returned trust lines to only
        // those shared between the two accounts. (This would have to be an argument to the method here and in i-c-client too.)
        const accountLinesRequest = {
            method: 'account_lines',
            params: [
                {
                    account: account,
                    ledger_index: 'validated',
                },
            ],
        };
        const axiosResponse = await this.submitRequest(accountLinesRequest);
        const accountLinesResponse = axiosResponse.data;
        return accountLinesResponse;
    }
}
exports.default = JsonRpcNetworkClient;
//# sourceMappingURL=json-rpc-network-client.js.map