"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const xrp_issued_currency_1 = __importDefault(require("./xrp-issued-currency"));
/**
 * An amount of currency on the XRP Ledger
 * @see: https://xrpl.org/basic-data-types.html#specifying-currency-amounts
 */
class XrpCurrencyAmount {
    /**
     * Note: Mutually exclusive fields - only drops XOR issuedCurrency should be set.
     *
     * @param drops An amount of XRP, specified in drops.
     * @param issuedCurrency An amount of an issued currency.
     */
    constructor(drops, issuedCurrency) {
        this.drops = drops;
        this.issuedCurrency = issuedCurrency;
    }
    /**
     * Constructs an XrpCurrencyAmount from a CurrencyAmount.
     *
     * @param currencyAmount a CurrencyAmount (protobuf object) whose field values will be used
     *                       to construct an XrpCurrencyAmount
     * @returns an XrpCurrencyAmount with its fields set via the analogous protobuf fields.
     * @see https://github.com/ripple/rippled/blob/develop/src/ripple/proto/org/xrpl/rpc/v1/amount.proto#L10
     */
    static from(currencyAmount) {
        // Mutually exclusive: either drops or issuedCurrency is set in an XRPCurrencyAmount
        const issuedCurrencyAmount = currencyAmount.getIssuedCurrencyAmount();
        const xrpAmount = currencyAmount.getXrpAmount();
        if (issuedCurrencyAmount !== undefined && xrpAmount === undefined) {
            const issuedCurrency = xrp_issued_currency_1.default.from(issuedCurrencyAmount);
            return new XrpCurrencyAmount(undefined, issuedCurrency);
        }
        if (xrpAmount !== undefined && issuedCurrencyAmount === undefined) {
            const drops = xrpAmount.getDrops();
            return new XrpCurrencyAmount(drops, undefined);
        }
        throw new __1.XrpError(__1.XrpErrorType.MalformedProtobuf, 'CurrencyAmount protobuf does not have an amount set.');
    }
}
exports.default = XrpCurrencyAmount;
//# sourceMappingURL=xrp-currency-amount.js.map